
// we use implements to specify the interface that ImplementationA implements
// ImplementationA is an instanceof InterfaceA (ie. we can cast objects of the type ImplementationA to InterfaceA)
public class ImplementationA implements InterfaceA {

	// we need to implement all methods of InterfaceA if we don't want to be ImplementationA abstract
	// (an example for this will follow shortly)
	@Override
	public void methodA() {
		// we can access the constants/final fields that we have defined in InterfaceA
		System.out.println(testDouble);
	}
	
	@Override
	public void methodB() {
		// let's be a bit naughty:
		final int[] b = new int[] { 1, 2, 3 };
		// b is of the type final int[]
		// does this mean that we can't change b?
		
		//b = new int[] { 1 };
		// doesn't work: The final local variable b cannot be assigned. It must be blank and not using a compound assignment
		
		// but - and this is actually something you rarely want:
		b[1] = 4;
		// works!
		
		// why?
		// final only makes the variable "final"
		// but b is only a reference to an int[] object
		// thus we can't assign a new reference to b
		// but we can still change the object b points to
		
		// thus:
		testIntArray[1] = 4;
		// and this is evil!
		
		// consequence:
		// declaring final reference variables is not a good idea
		// especially since everybody can access testIntArray because it's public
	}		
}