import java.util.ArrayList;
import java.util.Iterator;

// we can implement multiple interfaces:
// this is something we can't do with abstract classes (or classes in general)
// this example is a bit stupid, except to show that you can implement multiple interfaces at once
// and to show how you can implement an iterable object

// we implement three interfaces:
//	InterfaceA for the fun of using it
//  Iterable<Character> to be able to use our object in foreach loops (see main below for an example) 
public class WrappedList implements Iterable<Character>, InterfaceA {
	// (note how I use private to protect from changes from outside the class)
	private ArrayList<Character> charList;
	
	public WrappedList() {
		charList = new ArrayList<Character>();
	}
	
	// methodA and methodB just add As and Bs respectively to the list 
	@Override
	public void methodA() {
		charList.add('A');
	}

	@Override
	public void methodB() {
		charList.add('B');
	}
	
	// we need to implement this method for the interface Iterable<Character>
	// (to see the declaration move the cursor to Iterable<Character> and press F3 in Eclipse)
	@Override
	public Iterator<Character> iterator() {
		// I'm kind of lazy here
		// - implementing an actual Iterator would require declaring another class
		// so I'm just going to "wrap" the charList and directly return its Iterator
		// (ArrayList<T> implements the interface Iterable<T>, too
		return charList.iterator();
	}
	
	public static void main(String[] args) {
		// it's time for an example now
		// so let's create a new object
		WrappedList myList = new WrappedList();
		
		// we can, of course, cast it to Iterable, if we want to:
		Iterable<Character> myListAsIterable = myList;
		// or as InterfaceA
		InterfaceA myListAsInterfaceA = myList;
		
		// but anyway: let's add some As and Bs
		myList.methodA();
		myList.methodB();
		myList.methodB();
		myList.methodA();
		
		// and now we're going to use foreach on this object
		// (remember we have just written this on our own and it's not a collection)
		for( char c : myList ) {
			System.out.print(c);
		}
		System.out.println();
		// wonderful, isn't it?
		// the output is: ABBA
		// :-)
	}
}
